//	Title: AudioGraph.java
//	Description: I[fBIp̃Ot
//	Version: 1.0
//	Date: 2004-4-16
//	Copyright 2004 by Ayumi Nakabayashi. All rights reserved.

import java.awt.Color;
import java.awt.Graphics2D;
import fri.swing.graph.AWTRenderer;
import fri.swing.graph.GraphRenderer;
import fri.swing.graph.GraphUtil;
import fri.swing.graph.PlotGraph;
import fri.swing.graph.XYDataFactory;

/**
 *	I[fBIp̃Ot
 *	@author Ayumi Nakabayashi
 *	@version 1.0 2004-4-16
 */
public class AudioGraph extends PlotGraph {
	int[] AGcol;

	/**
	 *	Ot쐬.
	 *	@param d f[^
	 *	@param xaxt x̌`("log")
	 *	@param xmin x̍ŏl
	 *	@param xmax x̍ől
	 *	@param xlab x̃x
	 *	@param yaxt y̌`("dB", "log", "dist")
	 *	@param ymin y̍ŏl
	 *	@param ymax y̍ől
	 *	@param ylab ỹx
	 */
	public AudioGraph(XYDataFactory d,
			String xaxt, double xmin, double xmax, String xlab,
			String yaxt, double ymin, double ymax, String ylab) {
		super(d);
		setType("n");
		setMar(new double[] { 3.6, 3.6, 1.0, 1.0 });
		setMgp(new double[] { 2.5, 0.8, 0.0 });
		setLineType(0);
		box("n");
		setXaxt("n");
		setYaxt("n");
		setXaxs("i");
		setYaxs("i");
		setPty((xaxt.equals("log")
		  && (yaxt.equals("log") || yaxt.equals("dist"))) ? "a" : "m");
		LogAxis xla = null;
		if (xaxt.equals("log")) {
			xla = new LogAxis(xmin, xmax);
			xmin = xla.min;
			xmax = xla.max;
			axis(1, xla.tick, null, 1.0, 11);
		}
		setXlim(xmin, xmax);
		if (yaxt.equals("log") || yaxt.equals("dist")) {
			LogAxis yla = new LogAxis(ymin, ymax, yaxt.equals("dist"));
			ymin = yla.min;
			ymax = yla.max;
			axis(2, yla.tick, null, 1.0, 11);
			axis(2, yla.at, null, 1.0);
			axis(2, yla.at, yla.lab);
		} else if (yaxt.equals("dB")) {
			int nmin = iround(Math.floor(ymin / 10.0));
			int nmax = iround(Math.ceil(ymax / 10.0));
			int n = nmax - nmin + 1;
			double[] at = new double[n];
			for (int i = 0; i < n; i++)
				at[i] = ymin + 10.0 * i;
			String[] lab = GraphUtil.makelab(at);
			axis(2, at, null, 1.0);
			axis(2, at, lab);
		}
		setYlim(ymin, ymax);
		if (xla != null) {
			axis(1, xla.at, null, 1.0);
			axis(1, xla.at, xla.lab);
		}
		setXlab(xlab);
		setYlab(ylab);

		int[] colsrc = new int[] { 1, 2, 3, 5, 6, 4, 7, 8, 9, 10 };
		AGcol = new int[d.ncol()];
		for (int i = 0; i < AGcol.length; i++)
			AGcol[i] = colsrc[i];
		lines(d, AGcol, "b");
		box("0");
	}

	/**
	 *	x ľܓ int ɂ.
	 *	@param x ľܓl
	 */
	static int iround(double x) {
		int sign = (x >= 0.0) ? 1 : -1;
		x = Math.abs(x);
		return sign * (int)(x + 0.5);
	}

	/**
	 *	i Ԗڂ̐FƐݒ肷.
	 *	@param i ԍ
	 */
	public void setColorAndLineType(GraphRenderer rd, int i) {
		rd.setColor(colortbl[AGcol[i % 10]]);
		rd.setLineType(lty[i % lty.length]);
	}

	/**
	 *	V_ǉ.
	 *	@param g2 OtBNXReLXg
	 *	@param x _xW
	 *	@param y _yW
	 *	@param c ԍ
	 */
	void newPoint(Graphics2D g2, double x, double y, int c) {
		AWTRenderer rd = new AWTRenderer(g2);
		setClip(rd);
		setColorAndLineType(rd, c);
		rd.circle(X.translate(x), Y.translate(y), 1.5);
	}

	/**
	 *	Vǉ.
	 *	@param g2 OtBNXReLXg
	 *	@param x1 n_xW
	 *	@param y1 n_yW
	 *	@param x2 I_xW
	 *	@param y2 I_yW
	 *	@param c ԍ
	 */
	void newLine(Graphics2D g2, double x1, double y1, double x2, double y2, int c) {
		AWTRenderer rd = new AWTRenderer(g2);
		setClip(rd);
		setColorAndLineType(rd, c);
		rd.line(X.translate(x1), Y.translate(y1),
				X.translate(x2), Y.translate(y2));
	}
}
