//	Title: FRADriver.java
//	Description: g͊ 
//	Version: 1.1
//	Date: 2004-6-19
//	Copyright 2004 by Ayumi Nakabayashi. All rights reserved.

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.util.Enumeration;
import java.util.Properties;

import javax.comm.CommPortIdentifier;
import javax.comm.NoSuchPortException;
import javax.comm.PortInUseException;
import javax.comm.SerialPort;
import javax.comm.SerialPortEvent;
import javax.comm.UnsupportedCommOperationException;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fri.util.Util;
import fri.util.StringTokenizer2;

/**
 *	VAʐM^CAEgO.
 */
class TimeoutException extends Exception {
	public TimeoutException() {
		super();
	}
}

/**
 *	g͊ 
 *	@author Ayumi Nakabayashi
 *	@version 1.1 2004/6/19<br>
 *	1.0 2004/4/16
 */
public class FRADriver {
	static final boolean DEBUG = true;

	/** I/O|[g */
	String port;
	/** VA|[g */
	SerialPort sport = null;
	/** VA|[g̓̓Xg[ */
	InputStream is = null;
	/** VA|[gւ̏o̓Xg[ */
	OutputStream os = null;
	/** ݐݒ肳Ăg */
	double f;

	/** p[^ݒp_CAO */
	JDialog dialog;
	/** VA|[gI */
	JComboBox portChooser;

	/** |[gI[vĂ邩? */
	boolean ready;

	/**
	 *	IuWFNg쐬.
	 */
	public FRADriver(Properties prop) {
		ready = false;
		port = prop.getProperty("FRA.SerialPort");
		System.out.println("FRADriver.<init>: port=" + port);
	}

	public FRADriver() {
		this(FRA.prop);
	}

	/**
	 *	FRAgp\ɂ.
	 */
	public void setup() {
		if (!ready)
			open();
	}

	/**
	 *	FRAڑĂ邩AmF.
	 *	@return FRA[g[hɂȂĂƂ true Ԃ.
	 */
	public boolean isready() {
		write("Init\r");
		try {
			read(3000);
		} catch (TimeoutException e) {
			return false;
		}
		FRA.sleep(500);
		setFrequency(1e3);
		return true;
	}

	/**
	 *	FRAgp\ɂ.
	 */
	public void open() {
		CommPortIdentifier portid = null;
		try {
			portid = CommPortIdentifier.getPortIdentifier(port);
		} catch (NoSuchPortException e) {
			throw new RuntimeException("No such port " + port);
		}
		try {
			sport = (SerialPort) portid.open("FRA", 5000);
		} catch (PortInUseException e) {
			throw new RuntimeException(e.getMessage());
		}
		int baudRate = 9600;
		int dataBits = SerialPort.DATABITS_8;
		int stopBits = SerialPort.STOPBITS_1;
		int parity = SerialPort.PARITY_NONE;
		try {
			sport.setSerialPortParams(baudRate, dataBits, stopBits, parity);
			sport.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
		} catch (UnsupportedCommOperationException e) {
			throw new RuntimeException(e.getMessage());
		}
		try {
			os = sport.getOutputStream();
			is = sport.getInputStream();
		} catch (IOException e) {
			throw new RuntimeException(e.getMessage());
		}
		ready = true;
	}

	/**
	 *	I[fBIAiCU̎gp߂.
	 */
	public void close() {
		if (sport != null) {
			sport.close();
			sport = null;
		}
		if (is != null) {
			try {
				is.close();
			} catch (IOException e) {
				throw new RuntimeException(e.getMessage());
			}
			is = null;
		}
		if (os != null) {
			try {
				os.close();
			} catch (IOException e) {
				throw new RuntimeException(e.getMessage());
			}
			os = null;
		}
		ready = false;
	}

	/**
	 *	Ugݒ肷.
	 *	@param f Ug[Hz]
	 */
	public void setFrequency(double f) {
		this.f = f;
		long lfreq = (long)f;
		write("SetFreq\r");
		write(lfreq + "\r");
		long sleepTime = Math.max(400, 1000L / (long)f);
		System.out.println("sleepTime="+sleepTime);
		FRA.sleep((int)sleepTime);
	}

	/**
	 *	d𓾂.
	 *	@param level 2`l̃x(dB)
	 */
	public void getLevels(double[] level) {
		int n = (f < 10.0) ? 1 : 5;
		level[0] = level[1] = 0.0;
		StringTokenizer2 st = null;
		for (int i = 0; i < n; i++) {
			write("GetLevel\r");
			String l;
			try {
				l = read(3000);
			} catch (TimeoutException e) {
				level[0] = Double.NaN;
				level[1] = Double.NaN;
				return;
			}
			st = new StringTokenizer2(l, ",");
			level[0] += Util.atof(st.nextToken().trim());
			level[1] += Util.atof(st.nextToken().trim());
		}
		level[0] /= n;
		level[1] /= n;
	}

	/** ݒ\ȍōg */
	static final double FREQ_MAX = Math.pow(2.0, 25.0);

	/**
	 *	ݒłgԂ.
	 *	@param f g[Hz]
	 */
	public double frequency(double f) {
		if (f < 1.0)
			return 1.0;
		if (f > FREQ_MAX)
			return FREQ_MAX;
		return round(f, 0);
	}

	/**
	 *	ݒłŏ̎gԂ.
	 */
	public double minFrequency() {
		return 10.0;
	}

	/**
	 *	ݒłő̎gԂ.
	 */
	public double maxFrequency() {
		return FREQ_MAX;
	}

	/**
	 *	@̖OԂ.
	 */
	public String getName() {
		return "FRA 8307";
	}

	/**
	 *	ݒp̃_CAOJ.
	 */
	public void showSetupDialog(JFrame f) {
		dialog = new JDialog(f,						// L
				FRA.gets("title.FRA"),				// ^Cg
				true);								// [_

		JPanel inputPanel = new JPanel();
		inputPanel.setLayout(new GridBagLayout());
		JLabel portLabel = new JLabel(FRA.gets("label.SerialPort")+": ");
		FRA.constrain(inputPanel, portLabel, 0, 0, 1, 1, GridBagConstraints.EAST, 2);

		portChooser = new JComboBox();
		if (port != null)
			portChooser.addItem(port);
		Enumeration en = CommPortIdentifier.getPortIdentifiers();
		while (en.hasMoreElements()) {
			CommPortIdentifier portId = (CommPortIdentifier) en.nextElement();
			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL
			  && !portId.getName().equals(port))
				portChooser.addItem(portId.getName());
		}
		FRA.constrain(inputPanel, portChooser, 1, 0, 1, 1, GridBagConstraints.WEST, 2);

		JPanel buttonPanel = new JPanel();
		JButton okButton = new JButton(FRA.gets("button.OK"));
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				port = (String) portChooser.getSelectedItem();
				FRA.prop.setProperty("FRA.SerialPort", port);
				dialog.dispose();
				close();
			}
		});
		buttonPanel.add(okButton);
		JButton cancelButton = new JButton(FRA.gets("button.Cancel"));
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialog.dispose();
			}
		});
		buttonPanel.add(cancelButton);

		Container dialogContentPane = dialog.getContentPane();
		dialogContentPane.add(inputPanel, BorderLayout.CENTER);
		dialogContentPane.add(buttonPanel, BorderLayout.SOUTH);
		dialog.pack();
		dialog.show();
	}

	/**
	 *	_ȉ n ʂŎľܓ.
	 *	@param x ľܓl.
	 *	@param n 
	 */
	public static double round(double x, int n) {
		double sign = 1.0;
		if (x < 0.0) {
			sign = -1.0;
			x = -x;
		}
		double fac = Math.pow(10.0, n);
		return sign * Math.round(x * fac) / fac;
	}

	/**
	 *	VA|[gɕo͂.
	 *	@param s o͂镶
	 */
	protected void write(String s) {
		if (DEBUG)
			System.out.println("-> " + s);
		int n = s.length();
		try {
			for (int i = 0; i < n; i++)
				os.write(s.charAt(i));
		} catch (IOException e) {
			throw new RuntimeException(e.getMessage());
		}
	}
	
	/**
	 *	VA|[g當ǂݍ.
	 *	@param ms ^CAEg(ms)
	 */
	protected String read(int ms) throws TimeoutException {
		StringBuffer sb = new StringBuffer();
		for (;;) {
			char c = readOne(ms);
			if (c == 10)
				break;
			if (c != 13)
				sb.append((char)c);
		}
		String result = new String(sb);
		if (DEBUG)
			System.out.println("<- " + result);
		return result;
	}

	/**
	 *	VA|[g1.
	 *	@param ms ^CAEg(ms)
	 */
	public char readOne(int ms) throws TimeoutException {
		long timeout = System.currentTimeMillis() + ms;
		for (;;) {
			try {
				if (is.available() > 0) {
					int c = is.read();
					if (c != -1)
						return (char) c;
				}
			} catch (IOException e) {
				throw new RuntimeException(e.getMessage());
			}
			if (System.currentTimeMillis() > timeout)
				throw new TimeoutException();
			FRA.sleep(1);
		}
	}
}
